@@ -27,14 +27,23 @@
 #include <stdio.h>
 #include <syslog.h>
 #include <stdarg.h>
+#include <values.h>
 
 #include "dataxfer.h"
 #include "readconfig.h"
 #include "utils.h"
 #include "telnet.h"
+#include "led.h"
+
+#ifdef HAVE_OPENIPMI
+#include <OpenIPMI/ipmi_conn.h>
+#include <OpenIPMI/ipmi_sol.h>
+#endif
 
 #define MAX_LINE_SIZE 256	/* Maximum line length in the config file. */
 
+#define PORT_BUFSIZE	64	/* Default data transfer buffer size */
+
 extern char *config_port;
 
 static int config_num = 0;
@@ -190,7 +199,7 @@ finish_longstr(void)
 
 /* Parse the incoming string, it may be on multiple lines. */
 static void
-handle_longstr(char *name, char *line, enum str_type type)
+handle_longstr(const char *name, const char *line, enum str_type type)
 {
     int line_len;
 
@@ -280,8 +289,10 @@ find_str(const char *name, enum str_type *type, unsigned int *len)
 	    char *rv;
 
 	    /* Note that longstrs can contain \0, so be careful in handling */
-	    *type = longstr->type;
-	    *len = longstr->length;
+	    if (type)
+		*type = longstr->type;
+	    if (len)
+		*len = longstr->length;
 	    rv = malloc(longstr->length + 1);
 	    if (!rv)
 		return NULL;
@@ -293,7 +304,7 @@ find_str(const char *name, enum str_type *type, unsigned int *len)
     return NULL;
 }
 
-static void
+void
 free_longstrs(void)
 {
     struct longstr_s *longstr;
@@ -325,7 +336,7 @@ struct tracefile_s
     struct tracefile_s *next;
 };
 
-#ifdef USE_RS485_FEATURE
+#if HAVE_DECL_TIOCSRS485
 struct rs485conf_s
 {
     char *name;
@@ -381,7 +392,7 @@ find_tracefile(const char *name)
     return NULL;
 }
 
-static void
+void
 free_tracefiles(void)
 {
     struct tracefile_s *tracefile;
@@ -395,7 +406,7 @@ free_tracefiles(void)
     }
 }
 
-#ifdef USE_RS485_FEATURE
+#if HAVE_DECL_TIOCSRS485
 /* All the RS485 configs in the system. */
 struct rs485conf_s *rs485confs = NULL;
 
@@ -468,7 +479,7 @@ find_rs485conf(const char *name)
     return NULL;
 }
 
-static void
+void
 free_rs485confs(void)
 {
     struct rs485conf_s *rs485conf;
@@ -480,6 +491,8 @@ free_rs485confs(void)
         free(rs485conf);
     }
 }
+#else
+void free_rs485confs(void) { }
 #endif
 
 static int
@@ -512,6 +525,231 @@ static struct absout syslog_eout = {
     .data = &lineno
 };
 
+struct enum_val
+{
+    char *str;
+    int val;
+};
+
+static int
+lookup_enum(struct enum_val *enums, const char *str, int len)
+{
+    while (enums->str != NULL) {
+	if (len == -1 && strcmp(enums->str, str) == 0)
+	    return enums->val;
+	if (strlen(enums->str) == len && strncmp(enums->str, str, len) == 0)
+	    return enums->val;
+	enums++;
+    }
+    return -1;
+}
+
+static struct enum_val speed_enums[] = {
+    { "300",	300 },
+    { "600",	600 },
+    { "1200",	1200 },
+    { "2400",	2400 },
+    { "4800",	4800 },
+    { "9600",	9600 },
+    { "19200",	19200 },
+    { "38400",	38400 },
+    { "57600",	57600 },
+    { "115200",	115200 },
+    { "230400",	230400 },
+    { "460800",	460800 },
+    { "500000",	500000 },
+    { "576000",	576000 },
+    { "921600",	921600 },
+    { "1000000",1000000 },
+    { "1152000",1152000 },
+    { "1500000",1500000 },
+    { "2000000",2000000 },
+    { "2500000",2500000 },
+    { "3000000",3000000 },
+    { "3500000",3500000 },
+    { "4000000",4000000 },
+    { NULL },
+};
+
+int
+speedstr_to_speed(const char *speed)
+{
+    return lookup_enum(speed_enums, speed, -1);
+}
+
+struct enum_val parity_enums[] = {
+    { "NONE", PARITY_NONE },
+    { "EVEN", PARITY_EVEN },
+    { "ODD", PARITY_ODD },
+    { "none", PARITY_NONE },
+    { "even", PARITY_EVEN },
+    { "odd", PARITY_ODD },
+    { NULL }
+};
+
+#ifdef HAVE_OPENIPMI
+struct enum_val shared_serial_alert_enums[] = {
+    { "fail",		ipmi_sol_serial_alerts_fail },
+    { "deferred", 	ipmi_sol_serial_alerts_deferred },
+    { "succeed", 	ipmi_sol_serial_alerts_succeed },
+    { NULL }
+};
+#endif
+
+enum parity_vals
+lookup_parity(const char *str)
+{
+    return lookup_enum(parity_enums, str, -1);
+}
+
+enum default_type { DEFAULT_INT, DEFAULT_BOOL, DEFAULT_ENUM };
+
+struct default_data
+{
+    const char *name;
+    enum default_type type;
+    int min;
+    int max;
+    struct enum_val *enums;
+    union {
+	int intval;
+    } val;
+    union {
+	int intval;
+    } def;
+};
+
+struct default_data defaults[] = {
+    /* serial device only */
+    { "stopbits",	DEFAULT_INT,	.min = 1, .max = 2, .def.intval = 1 },
+    { "databits",	DEFAULT_INT,	.min = 5, .max = 8, .def.intval = 8 },
+    { "parity",		DEFAULT_ENUM,	.enums = parity_enums,
+					.def.intval = PARITY_NONE },
+    { "xonxoff",	DEFAULT_BOOL,	.def.intval = 0 },
+    { "rtscts",		DEFAULT_BOOL,	.def.intval = 0 },
+    { "local",		DEFAULT_BOOL,	.def.intval = 0 },
+    { "hangup_when_done", DEFAULT_BOOL,	.def.intval = 0 },
+    /* Serial port and SOL */
+    { "speed",		DEFAULT_ENUM,	.enums = speed_enums,
+					.def.intval = 9600 },
+    { "nobreak",	DEFAULT_BOOL,	.def.intval = 0 },
+    /* All port types */
+    { "remctl",		DEFAULT_BOOL,	.def.intval = 0 },
+    { "telnet_brk_on_sync",DEFAULT_BOOL,.def.intval = 0 },
+    { "kickolduser",	DEFAULT_BOOL,	.def.intval = 0 },
+    { "chardelay",	DEFAULT_BOOL,	.def.intval = 1 },
+    { "chardelay-scale",DEFAULT_INT,	.min=1, .max=1000, .def.intval = 20 },
+    { "chardelay-min",	DEFAULT_INT,	.min=1, .max=100000,
+					.def.intval = 1000 },
+    { "chardelay-max",	DEFAULT_INT,	.min=1, .max=100000,
+					.def.intval = 20000 },
+    { "dev-to-tcp-bufsize", DEFAULT_INT,.min=1, .max=65536,
+					.def.intval = PORT_BUFSIZE },
+    { "tcp-to-dev-bufsize", DEFAULT_INT,.min=1, .max=65536,
+					.def.intval = PORT_BUFSIZE },
+#ifdef HAVE_OPENIPMI
+    /* SOL only */
+    { "authenticated",	DEFAULT_BOOL,	.def.intval = 1 },
+    { "encrypted",	DEFAULT_BOOL,	.def.intval = 1 },
+    { "ack-timeout",	DEFAULT_INT,	.min=1, .max=INT_MAX,
+					.def.intval = 1000000 },
+    { "ack-retries",	DEFAULT_INT,	.min=1, .max=INT_MAX,
+					.def.intval = 10 },
+    { "shared-serial-alert", DEFAULT_ENUM, .enums = shared_serial_alert_enums,
+				   .def.intval = ipmi_sol_serial_alerts_fail },
+    { "deassert_CTS_DCD_DSR_on_connect", DEFAULT_BOOL, .def.intval = 0 },
+#endif
+    { NULL }
+};
+
+
+static void
+setup_defaults(void)
+{
+    int i;
+
+    for (i = 0; defaults[i].name; i++) {
+	defaults[i].val.intval = defaults[i].def.intval;
+    }
+}
+
+int
+find_default_int(const char *name)
+{
+    int i;
+
+    for (i = 0; defaults[i].name; i++) {
+	if (strcmp(defaults[i].name, name) == 0)
+	    return defaults[i].val.intval;
+    }
+    abort();
+}
+
+static void
+handle_new_default(const char *name, const char *str)
+{
+    int i, val, len;
+    char *end;
+    const char *s;
+
+    while (isspace(*str))
+	str++;
+    s = str;
+    while (!isspace(*s) && *s != '\0')
+	s++;
+    if (s == str) {
+	syslog(LOG_ERR, "No default value on %d", lineno);
+	return;
+    }
+    len = s - str;
+
+    for (i = 0; defaults[i].name; i++) {
+	if (strcmp(defaults[i].name, name) != 0)
+	    continue;
+
+	switch (defaults[i].type) {
+	case DEFAULT_INT:
+	    val = strtoul(str, &end, 10);
+	    if (end != s) {
+		syslog(LOG_ERR, "Invalid integer value on %d", lineno);
+		return;
+	    }
+	    if (val < defaults[i].min || val > defaults[i].max) {
+		syslog(LOG_ERR, "Integer value out of range on %d, "
+		       "min is %d, max is %d",
+		       lineno, defaults[i].min, defaults[i].max);
+		return;
+	    }
+	    defaults[i].val.intval = val;
+	    break;
+
+	case DEFAULT_BOOL:
+	    val = strtoul(str, &end, 10);
+	    if (end == s)
+		defaults[i].val.intval = !!val;
+	    else if (len == 4 && ((strncmp(str, "true", 4) == 0) ||
+				  (strncmp(str, "TRUE", 4) == 0)))
+		defaults[i].val.intval = 1;
+	    else if (len == 5 && ((strncmp(str, "false", 5) == 0) ||
+				  (strncmp(str, "FALSE", 5) == 0)))
+		defaults[i].val.intval = 0;
+	    else
+		syslog(LOG_ERR, "Invalid integer value on %d", lineno);
+	    break;
+
+	case DEFAULT_ENUM:
+	    val = lookup_enum(defaults[i].enums, str, len);
+	    if (val == -1) {
+		syslog(LOG_ERR, "Invalid enumeration value on %d", lineno);
+		return;
+	    }
+	    defaults[i].val.intval = val;
+	    break;
+	}
+	return;
+    }
+}
+
 void
 handle_config_line(char *inbuf)
 {
@@ -570,7 +808,7 @@ handle_config_line(char *inbuf)
 	return;
     }
 
-#ifdef USE_RS485_FEATURE
+#if HAVE_DECL_TIOCSRS485
     if (startswith(inbuf, "RS485CONF", &strtok_data)) {
         char *name = strtok_r(NULL, ":", &strtok_data);
         char *str = strtok_r(NULL, "\n", &strtok_data);
@@ -635,6 +873,54 @@ handle_config_line(char *inbuf)
 	return;
     }
 
+    if (startswith(inbuf, "DEVICE", &strtok_data)) {
+	char *name = strtok_r(NULL, ":", &strtok_data);
+	char *str = strtok_r(NULL, "\n", &strtok_data);
+	if (name == NULL) {
+	    syslog(LOG_ERR, "No device name given on line %d", lineno);
+	    return;
+	}
+	handle_longstr(name, str, DEVNAME);
+	return;
+    }
+
+    if (startswith(inbuf, "DEFAULT", &strtok_data)) {
+	char *name = strtok_r(NULL, ":", &strtok_data);
+	char *str = strtok_r(NULL, "\n", &strtok_data);
+	if (name == NULL) {
+	    syslog(LOG_ERR, "No default name given on line %d", lineno);
+	    return;
+	}
+	handle_new_default(name, str);
+	return;
+    }
+
+    if (startswith(inbuf, "ROTATOR", &strtok_data)) {
+	char *name = strtok_r(NULL, ":", &strtok_data);
+	char *str = strtok_r(NULL, "\n", &strtok_data);
+	if (name == NULL) {
+	    syslog(LOG_ERR, "No rotator name given on line %d", lineno);
+	    return;
+	}
+	add_rotator(name, str, lineno);
+	return;
+    }
+
+    if (startswith(inbuf, "LED", &strtok_data)) {
+	char *name = strtok_r(NULL, ":", &strtok_data);
+	char *str = strtok_r(NULL, "\n", &strtok_data);
+	if (name == NULL) {
+	    syslog(LOG_ERR, "No LED name given on line %d", lineno);
+	    return;
+	}
+	if ((str == NULL) || (strlen(str) == 0)) {
+	    syslog(LOG_ERR, "No LED given on line %d", lineno);
+	    return;
+	}
+	handle_led(name, str, lineno);
+	return;
+    }
+
     comma = strchr(inbuf, ',');
     if (comma) {
 	if (!strtok_r(comma, ":", &strtok_data)) {
@@ -695,13 +981,16 @@ readconfig(char *filename)
 	return -1;
     }
 
+    setup_defaults();
     free_longstrs();
     free_tracefiles();
-#ifdef USE_RS485_FEATURE
+#if HAVE_DECL_TIOCSRS485
     free_rs485confs();
 #endif
+    free_leds();
 
     config_num++;
+    free_rotators();
 
     while (fgets(inbuf, MAX_LINE_SIZE, instream) != NULL) {
 	int len = strlen(inbuf);
